package com.uas.erp.manage.client.api.v1;

import com.uas.erp.manage.client.entity.Artifact;
import com.uas.erp.manage.client.service.ArtifactService;
import com.uas.erp.manage.client.web.ResponseWrap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

/**
 * Created by Pro1 on 2017/6/19.
 */
@RestController
@RequestMapping(path = "/v1/artifact")
public class ArtifactControllerV1 {

    @Autowired
    private ArtifactService artifactService;

    /**
     * 所有信息
     * @return
     */
    @GetMapping(path = "/list")
    public ResponseEntity getArtifacts() {
        return ResponseWrap.ok(artifactService.getArtifacts());
    }

    /**
     * 查看仓库信息
     * @param id
     * @return
     */
    @GetMapping(path = "/{id}")
    public ResponseEntity getArtifact(@PathVariable("id") long id) throws Exception{
        return ResponseWrap.ok(artifactService.getArtifact(id));
    }

    /**
     * 查看仓库信息
     * @param groupId
     * @param artifactId
     * @return
     */
    @GetMapping(path = "/{groupId}/{artifactId}")
    public ResponseEntity getArtifact(@PathVariable("groupId") String groupId,@PathVariable("artifactId") String artifactId) throws Exception{
        return ResponseWrap.ok(artifactService.getArtifact(groupId, artifactId));
    }

    /**
     * 详细版本信息
     * @param groupId
     * @param artifactId
     * @return
     */
    @GetMapping(path = "/{groupId}/{artifactId}/items")
    public ResponseEntity getArtifactItems(@PathVariable("groupId") String groupId, @PathVariable("artifactId") String artifactId) {
        return ResponseWrap.ok(artifactService.getArtifactItems(groupId, artifactId));
    }

    /**
     * 更新到最新版本
     * @param id
     * @return
     */
    @GetMapping(path = "/{id}/pull")
    public ResponseEntity updateArtifactVersion(@PathVariable("id") long id) throws Exception{
        Artifact artifact = artifactService.getArtifact(id);
        Assert.notNull(artifact);
        artifactService.update(artifact);
        return ResponseWrap.ok();
    }

    /**
     * 详细版本信息
     * @param id
     * @return
     */
    @GetMapping(path = "/{id}/items")
    public ResponseEntity getArtifactItems(@PathVariable("id") long id) {
        return ResponseWrap.ok(artifactService.getArtifactItems(id));
    }

    /**
     * 保存
     * @param artifact
     * @return
     */
    @PostMapping
    public ResponseEntity saveArtifact(Artifact artifact) {
        artifactService.safetySave(artifact);
        return ResponseWrap.ok();
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @DeleteMapping(path = "/{id}")
    public ResponseEntity deleteArtifact(@PathVariable("id") long id) {
        Artifact artifact = artifactService.getArtifact(id);
        Assert.notNull(artifact);
        artifactService.delete(artifact);
        return ResponseWrap.ok();
    }

}
