package com.uas.erp.manage.client.api.v1;

import com.uas.erp.manage.client.entity.Container;
import com.uas.erp.manage.client.service.ContainerService;
import com.uas.erp.manage.client.web.ResponseWrap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

/**
 * Created by Pro1 on 2017/6/19.
 */
@RestController
@RequestMapping(path = "/v1/container")
public class ContainerControllerV1 {

    @Autowired
    private ContainerService containerService;

    /**
     * 输入容器路径时检测
     * @param path
     * @return
     * @throws Exception
     */
    @GetMapping(path = "/check")
    public ResponseEntity checkByPath(@RequestParam String path) throws Exception{
        return ResponseWrap.ok(containerService.getContainerByPath(path));
    }

    /**
     * 保存
     * @param container
     * @return
     */
    @PostMapping
    public ResponseEntity save(Container container) throws Exception {
        containerService.safetySave(container);
        return ResponseWrap.ok();
    }

    @GetMapping(path = "/list")
    public ResponseEntity getAll() {
        return ResponseWrap.ok(containerService.findAll());
    }

    /**
     * 获取容器
     * @param containerId
     * @return
     */
    @GetMapping(path = "/{containerId}")
    public ResponseEntity get(@PathVariable("containerId") long containerId) {
        return ResponseWrap.ok(containerService.findOne(containerId));
    }

    /**
     * 删除容器
     * @param containerId
     * @return
     */
    @DeleteMapping(path = "/{containerId}")
    public ResponseEntity delete(@PathVariable("containerId") long containerId) {
        Container container = containerService.findOne(containerId);
        Assert.notNull(container);
        containerService.delete(container);
        return ResponseWrap.ok();
    }

    /**
     * 安装容器环境
     * @param containerId
     * @return
     */
    @RequestMapping(path = "/{containerId}/install")
    public ResponseEntity install(@PathVariable("containerId") long containerId) {
        Container container = containerService.findOne(containerId);
        Assert.notNull(container);
        return ResponseWrap.ok();
    }

    /**
     * 停止容器
     * @param containerId
     * @return
     */
    @RequestMapping(path = "/{containerId}/stop")
    public ResponseEntity stop(@PathVariable("containerId") long containerId) throws Exception {
        Container container = containerService.findOne(containerId);
        Assert.notNull(container);
        containerService.stop(container);
        return ResponseWrap.ok();
    }

    /**
     * 启动容器
     * @param containerId
     * @return
     */
    @RequestMapping(path = "/{containerId}/start")
    public ResponseEntity start(@PathVariable("containerId") long containerId) throws Exception {
        Container container = containerService.findOne(containerId);
        Assert.notNull(container);
        containerService.start(container);
        return ResponseWrap.ok();
    }

    /**
     * 重启容器
     * @param containerId
     * @return
     */
    @RequestMapping(path = "/{containerId}/restart")
    public ResponseEntity restart(@PathVariable("containerId") long containerId) throws Exception{
        Container container = containerService.findOne(containerId);
        Assert.notNull(container);
        containerService.restart(container);
        return ResponseWrap.ok();
    }

}
