package com.uas.erp.manage.client.api.v1;

import com.uas.erp.manage.client.entity.Artifact;
import com.uas.erp.manage.client.entity.ArtifactItem;
import com.uas.erp.manage.client.entity.Project;
import com.uas.erp.manage.client.service.ArtifactService;
import com.uas.erp.manage.client.service.ProjectService;
import com.uas.erp.manage.client.web.ResponseWrap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.util.List;

/**
 * Created by Pro1 on 2017/6/19.
 */
@RestController
@RequestMapping(path = "/v1/project")
public class ProjectControllerV1 {

    @Autowired
    private ProjectService projectService;

    @Autowired
    private ArtifactService artifactService;

    @GetMapping(path = "/list")
    public ResponseEntity getAll(String groupId, String artifactId) {
        List<Project> projects = null;
        if (null != groupId && null != artifactId) {
            Artifact artifact = artifactService.getArtifact(groupId, artifactId);
            projects = projectService.findByArtifactId(artifact.getId());
        } else {
            projects = projectService.findAll();
        }
        return ResponseWrap.ok(projects);
    }

    @PostMapping
    public ResponseEntity save(Project project) {
        projectService.save(project);
        return ResponseWrap.ok();
    }

    @GetMapping(path = "/{id}")
    public ResponseEntity get(@PathVariable("id") long id) {
        return ResponseWrap.ok(projectService.get(id));
    }

    @DeleteMapping(path = "/{id}")
    public ResponseEntity delete(@PathVariable("id") long id) throws Exception{
        projectService.delete(id);
        return ResponseWrap.ok();
    }

    /**
     * 部署{artifact}相关全部项目
     * @param groupId
     * @param artifactId
     * @param version
     * @throws Exception
     */
    @RequestMapping(path = "/deploy")
    public ResponseEntity deployProject(final String groupId, final String artifactId, String version) throws Exception{
        Artifact artifact = artifactService.getArtifact(groupId, artifactId);
        Assert.notNull(artifact);
        // 检测并更新仓库
        ArtifactItem artifactItem = null;
        if (!StringUtils.isEmpty(version))
            artifactItem = artifactService.updateToVersion(artifact, version);// 指定版本
        else
            artifactItem = artifactService.update(artifact);// 最新版本
        List<Project> projects = projectService.findByArtifactId(artifact.getId());
        if (!CollectionUtils.isEmpty(projects)) {
            for (Project project:projects) {
                projectService.deploy(project, artifactItem);
            }
        }
        return ResponseWrap.ok();
    }

    /**
     * 指定项目及部署版本
     * @param projectId
     * @param artifactItemId
     */
    @RequestMapping(path = "/{projectId}/deploy")
    public ResponseEntity deployProject(@PathVariable("projectId") long projectId, @RequestParam long artifactItemId) throws Exception{
        Project project = projectService.findOne(projectId);
        Assert.notNull(project);
        ArtifactItem artifactItem = artifactService.getArtifactItem(artifactItemId);
        Assert.notNull(artifactItem);
        projectService.deploy(project, artifactItem);
        return ResponseWrap.ok();
    }

    /**
     * 查找日志
     * @return
     */
    @GetMapping(path = "/{projectId}/deploy/log")
    public ResponseEntity getDeployLog(@PathVariable("projectId") long projectId) {
        return ResponseWrap.ok(projectService.getDeployLogs(projectId));
    }

}
