package com.uas.erp.manage.client.init;

import com.alibaba.fastjson.JSON;
import com.uas.erp.manage.client.entity.Artifact;
import com.uas.erp.manage.client.entity.Client;
import com.uas.erp.manage.client.entity.Setting;
import com.uas.erp.manage.client.service.*;
import com.uas.erp.manage.client.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * Created by Pro1 on 2017/6/26.
 */
@Component
public class DefaultClientSetting implements CommandLineRunner {

    @Autowired
    private ClientService clientService;

    @Autowired
    private ArtifactService artifactService;

    @Autowired
    private SettingService settingService;

    @Autowired
    private ContainerService containerService;

    @Autowired
    private ProjectService projectService;

    @Autowired
    private Environment environment;

    @Override
    public void run(String... strings) throws Exception {
        if (null == clientService.getClient()) {
            // 客户端
            Client client = new Client();
            // 所有客户的ID唯一
            client.setId(StringUtil.uuid());
            client.setInit(isDev());
            clientService.save(client);

            // 清除可能存在的旧数据
            settingService.deleteAll();
            projectService.deleteAll();
            containerService.deleteAll();
            artifactService.deleteAll();

            // 默认数据
            String settingsStr = StringUtil.copyResource("init/setting.json");
            if (!StringUtils.isEmpty(settingsStr)) {
                List<Setting> settings = JSON.parseArray(settingsStr, Setting.class);
                settingService.save(settings);
            }
            String artifactsStr = StringUtil.copyResource("init/artifact.json");
            if (!StringUtils.isEmpty(artifactsStr)) {
                List<Artifact> artifacts = JSON.parseArray(artifactsStr, Artifact.class);
                artifactService.save(artifacts);
            }
        }
    }

    private boolean isDev() {
        for (String env : environment.getActiveProfiles()) {
            if ("dev".equalsIgnoreCase(env)) {
                return true;
            }
        }
        return false;
    }
}
