package com.uas.erp.manage.client.service;

import com.uas.erp.manage.client.domain.ClientContainerInfo;
import com.uas.erp.manage.client.domain.ClientInfo;
import com.uas.erp.manage.client.domain.ClientProjectInfo;
import com.uas.erp.manage.client.entity.Artifact;
import com.uas.erp.manage.client.entity.Client;
import com.uas.erp.manage.client.entity.Container;
import com.uas.erp.manage.client.entity.Project;
import com.uas.erp.manage.client.repository.ClientRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/6/23.
 */
@Service
public class ClientService {

    @Autowired
    private ClientRepository clientRepository;

    @Autowired
    private ProjectService projectService;

    @Autowired
    private ArtifactService artifactService;

    @Autowired
    private ContainerService containerService;

    public Client getClient() {
        List<Client> clients = clientRepository.findAll();
        return CollectionUtils.isEmpty(clients) ? null : clients.get(0);
    }

    public void save(Client client) {
        clientRepository.save(client);
    }

    public void safetySave(Client client) {
        Client oldOne = getClient();
        if (null != oldOne) {
            oldOne.setName(client.getName());
            oldOne.setBasePath(client.getBasePath());
        }
        clientRepository.save(oldOne);
    }

    public void initClient() {
        Client client = getClient();
        client.setInit(true);
        client.setInitTime(System.currentTimeMillis());
        clientRepository.save(client);
    }

    public ClientInfo getClientInfo() {
        Client client = getClient();
        if (null != client && client.isInit()) {
            ClientInfo info = new ClientInfo();
            info.setClientId(client.getId());
            info.setBasePath(client.getBasePath());
            info.setName(client.getName());

            List<Project> projects = projectService.findAll();
            List<ClientProjectInfo> projectInfos = new ArrayList<ClientProjectInfo>();
            if (!CollectionUtils.isEmpty(projects)) {
                for (Project project:projects) {
                    Artifact artifact = artifactService.getArtifact(project.getArtifactId());
                    projectInfos.add(new ClientProjectInfo(artifact, project));
                }
            }
            info.setProjects(projectInfos);

            List<Container> containers = containerService.findAll();
            List<ClientContainerInfo> containerInfos = new ArrayList<ClientContainerInfo>();
            if (!CollectionUtils.isEmpty(containers)) {
                for (Container container:containers) {
                    containerInfos.add(new ClientContainerInfo(container));
                }
            }
            info.setContainers(containerInfos);
            return info;
        }
        return null;
    }

}
