package com.uas.erp.manage.client.service;

import com.uas.erp.manage.client.entity.*;
import com.uas.erp.manage.client.plugins.ProjectResourceReplacePlugin;
import com.uas.erp.manage.client.repository.ProjectDeployLogRepository;
import com.uas.erp.manage.client.repository.ProjectRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Pro1 on 2017/6/19.
 */
@Service
public class ProjectService {

    @Autowired
    private ProjectRepository projectRepository;

    @Autowired
    private ArtifactService artifactService;

    @Autowired
    private LocalRepositoryService localRepository;

    @Autowired
    private ContainerService containerService;

    @Autowired
    private ProjectDeployLogRepository projectDeployLogRepository;

    private Map<Long, String> projectDeployTasks = new HashMap<Long, String>();

    public void save(Project project) {
        projectRepository.save(project);
    }

    public Project get(long id) {
        return projectRepository.findOne(id);
    }

    @Transactional
    public void delete(long id) throws Exception{
        Project project = projectRepository.findOne(id);
        if (null != project) {
            Container container = containerService.findOne(project.getContainerId());
            if (null != container)
                containerService.clearByContext(container, project.getContextName());
            projectDeployLogRepository.deleteByProjectId(id);
            projectRepository.delete(project);
        }
    }

    public void deleteAll() {
        projectDeployLogRepository.deleteAll();
        projectRepository.deleteAll();
        this.projectDeployTasks = new HashMap<Long, String>();
    }

    public Project findOne(long id) {
        return projectRepository.findOne(id);
    }

    public List<Project> findAll() {
        return projectRepository.findAll();
    }

    public List<Project> findByArtifactId(long artifactId) {
        return projectRepository.findByArtifactId(artifactId);
    }

    public List<Project> findByContainerId(long containerId) {
        return projectRepository.findByContainerId(containerId);
    }

    public void deploy(Project project, ArtifactItem artifactItem) throws Exception{
        String taskName = projectDeployTasks.get(project.getId());
        if (null != taskName)
            throw new Exception(String.format("项目正在部署！执行步骤为【%s】", taskName));
        try {
            projectDeployTasks.put(project.getId(), "打包");
            final Artifact artifact = artifactService.getArtifact(artifactItem.getArtifactId());
            List<ArtifactItem> allItems = artifactService.getArtifactItems(artifact.getId());
            // 打包
            File buildFile = null;
            if (Artifact.RepoType.DIFF.equals(Artifact.RepoType.valueOf(artifact.getRepoType()))) {
                buildFile = localRepository.buildByManifest(artifact, allItems, artifactItem, project.getContextName(),
                        new ProjectResourceReplacePlugin(project, localRepository));
            } else {
                buildFile = localRepository.buildMavenFile(artifact, artifactItem, project.getContextName(),
                        new ProjectResourceReplacePlugin(project, localRepository));
            }
            // 部署
            Container container = containerService.findOne(project.getContainerId());
            Assert.notNull(container);
            projectDeployTasks.put(project.getId(), "停止容器");
            containerService.stop(container);
            containerService.clearByContext(container, project.getContextName());
            containerService.deployByFile(container, buildFile);
            projectDeployTasks.put(project.getId(), "启动容器");
            containerService.start(container);
            projectDeployTasks.put(project.getId(), "记录日志");
            project.setDeployTime(new Date().getTime());
            project.setDeployVersion(artifactItem.getVersion());
            save(project);
            // 日志
            projectDeployLogRepository.save(new ProjectDeployLog(project, artifactItem));
        } finally {
            projectDeployTasks.remove(project.getId());
        }
    }

    public Map<Long, String> getDeployTasks() {
        return projectDeployTasks;
    }

    public String getDeployTask(long projectId) {
        return projectDeployTasks.get(projectId);
    }

    public List<ProjectDeployLog> getDeployLogs(long projectId) {
        return projectDeployLogRepository.findByProjectIdOrderByDeployTimeDesc(projectId);
    }

}
